#include <iostream.h>
#include <iomanip.h>
#include <math.h>
#include "lfout.h"

//------------------------------------------------------------------------------
// Setzen der Klassenvariablen gem der an die Klasse LFOUT bergebenen Werte -
//------------------------------------------------------------------------------
void lfout::set (long double lf, int vks, int prec)
{
	//---------------------------------------------------------------------------
	// Setzen der LONG DOUBLE-Variablen auf den im Parameter bergebenen Wert   -
	//---------------------------------------------------------------------------
	llf = lf;

	//---------------------------------------------------------------------------
	// Setzen der Nachkomma-Variable, Korrektur illegaler Werte (kleiner -1)    -
   // -1 bedeutet Standardwert (d.h. 6 Nachkommastellen)                       -
   //  0 bedeutet keine Nachkommastellen und ist daher ein gltiger Wert       -
	// Da damit die Anzahl der Nachkommastellen feststeht, kann diese in NK     -
   // gespeichert werden                                                       -
	//---------------------------------------------------------------------------
	if (prec < 0)
   {
   	prec = -1;
   }

	NK = prec;

	//---------------------------------------------------------------------------
	// Setzen der Vorkomma-Variable, Korrektur illegaler Werte (kleiner -1)     -
   // -1 bedeutet Standardwert (d.h. die bentigte Anzahl von Vorkommastellen) -
   // die Anzahl der Vorkommastellen ist nur dann von Interesse, wenn die      -
   // Anzahl der bentigten Stellen KLEINER ist als die Anzahl der gewnschten -
   // Stellen.                                                                 -
	//---------------------------------------------------------------------------
	if (vks  < 0)
   {
   	vks = -1;
   }

	//---------------------------------------------------------------------------
	// Wenn Nachkommastellen gewnscht sind und die angegebene Anzahl GRSSER   -
   // ist als die GESAMTZAHL der gewnschten Stellen also z.B. lfout(x,3,9)    -
   // und die Gesamtanzahl der Stellen wurde bergeben (ungleich -1)           -
   // dann immer die bentigte Anzahl an Vorkommastellen ausgeben (wie -1)     -
	//---------------------------------------------------------------------------
	if (prec > 0)
   {
		if ((prec >= vks) && (vks != -1))
      {
			vks = -1;
      }
   }

	//---------------------------------------------------------------------------
	// Wenn Nachkommastellen gewnscht sind und die angegebene Gesamtanzahl an  -
   // Stellen wurde bergeben (ungleich -1), dann ist die Anzahl der Vorkomma- -
   // stellen zu berechnen. Sonst entspricht die Anzahl der Vorkommastellen    -
   // der Anzahl der Gesamtstellen                                             -
	//---------------------------------------------------------------------------
	if ((prec > 0) && (vks != -1))
   {
		VK = vks - prec - 1;
   }
	else
   {
		VK = vks;
   }
}

//------------------------------------------------------------------------------
// Formatierte Ausgabe eines LFOUT-Objekts in einem OSTREAM                    -
//------------------------------------------------------------------------------
ostream& operator << (ostream& out, lfout& f)
{
	long double rundung = 0.0;
	//---------------------------------------------------------------------------
	// Wert der Vorkommastellen werden durch einfaches Abschneiden der Nach-    -
   // kommastellen gebildet                                                    -
	//---------------------------------------------------------------------------
	long double tmpvk = floor (f.llf);

	//---------------------------------------------------------------------------
	// Nachkommastellen erstmal auf Null setzen                                 -
	//---------------------------------------------------------------------------
	long double tmpnk = 0.0;

	//---------------------------------------------------------------------------
	// Keine Exponentialausgaben zulassen! Dazu mu ein Flag gesetzt werden,    -
   // der vorherige Wert des Flags wird zwischengespeichert und vor Funktions- -
   // ende restauriert                                                         -
	//---------------------------------------------------------------------------
	long oldflag = out.setf(ios::fixed, ios::floatfield);

	//---------------------------------------------------------------------------
   // Gleiches gilt fr die Ausgabeprzision, die hier auf Null (keine Nach-   -
   // kommastellen) gesetzt wird, die Nachkommastellen machen wir selbst.      -
	//---------------------------------------------------------------------------
	int  oldprec = out.precision(0);

	//---------------------------------------------------------------------------
	// Wenn definierte Anzahl von Nachkommastellen gewnscht ist, dann diese    -
   // in Vorkommastellen einer anderen Variablen umwandeln                     -
   // Die Addition von +0.1 vermeidet Rundungsfehler bei der Subtraktion       -
	// ist der Wert fr Nachkommastellen gleich -1, dann 6 Stellen ausgeben     -
	//---------------------------------------------------------------------------
	if (f.NK != -1)
	{
	  tmpnk = ((f.llf - tmpvk) * (pow10 (f.NK)));
   }
	else
   {
	  tmpnk = ((f.llf - tmpvk) * (pow10 (6)));
   }
	rundung = tmpnk - floor (tmpnk);
	tmpnk = floor (tmpnk);

	if (rundung >= 0.5)
	{
		tmpnk += 1.0;
	}

	if (tmpnk / pow10(f.NK) == 1.0)
	{
		tmpvk += 1.0;
		tmpnk  = 0.0;
	}
	//---------------------------------------------------------------------------
	// gibt es weder fr Vor- noch fr Nachkommastellen eine Vorgabe, dann die  -
   // Flags wiederherstellen und ganz normal ausgeben                          -
	//---------------------------------------------------------------------------
	if ((f.NK == -1) && (f.VK == -1))
	{
		out.precision (oldprec);
		out.setf (oldflag, ios::floatfield);
		out << f.llf;
		return (out);
	}

	//---------------------------------------------------------------------------
	// Ausgabe der Vorkommastellen gem Anzahl der gewnschten Stellen oder    -
   // die bentigten Stellen (ELSE-Fall)                                       -
	//---------------------------------------------------------------------------
	if ((f.VK > 0) && (f.NK != -1))
	{
		out << setw (f.VK) << tmpvk;
	}
	else
	{
		out << tmpvk;
	}

	//---------------------------------------------------------------------------
	// Ausgabe der Nachkommastellen gem Anzahl der gewnschten Stellen oder   -
   // die bentigten Stellen (ELSE-Fall)                                       -
   // Dazu mu man mit Nullen auffllen!                                       -
	//---------------------------------------------------------------------------
	if (f.NK > 0)
	{
		char fillbuff = out.fill ('0');
		out << "." << setw(f.NK) << tmpnk;
		out.fill (fillbuff);
	}
	if (f.NK < 0)
   {
		out << "." << tmpnk;
   }

	//---------------------------------------------------------------------------
   // Flags wiederherstellen                                                   -
	//---------------------------------------------------------------------------
	out.precision (oldprec);
	out.setf (oldflag, ios::floatfield);

	return (out);
}

